from typing import List

from RsInstrument import *
from GFM315_Px_tests.General.GFM315_utilities import *


class PxTestsBase:

    def __init__(self, resource_name: str):
        try:
            self.io = RsInstrument(resource_name, True, True)
        except ResourceError as e:
            raise RsException(e.args[0])
        self.io.visa_timeout = 12000

    def instrument_settings(self, trigger_config, marker_config):
        """
        This function sets instrument related settings.
        :param trigger_config: Trigger configuration:  Armed Auto ('AAUT') or unchanged ('UNCH')
        :param marker_config: Marker configuration: Radio frame start ('FRAM') or unchanged ('UNCH')
        :return: Instrument status
        """
        if trigger_config is not None:
            self.io.write_str(f':BB:NR5G:TCW:TRIG {trigger_config}')
        if marker_config is not None:
            self.io.write_str(f':BB:NR5G:TCW:MARK {marker_config}')
        return self.io.query_str(':syst:err?')

    def antenna_settings(self, tx_antennas, rx_antennas):
        """
        This function sets the antenna related settings.
        :param tx_antennas: Number of transmit antennas
        :param rx_antennas: Number of receive antennas
        :return: Instrument status
        """
        if tx_antennas is not None:
            self.io.write_str(f':BB:NR5G:TCW:ANT:TXAN ANT{tx_antennas}')
        if rx_antennas is not None:
            self.io.write_str(f':BB:NR5G:TCW:ANT:RXAN ANT{rx_antennas}')
        return self.io.query_str(':syst:err?')

    def feedback_settings(self, feedback_mode, serial_rate, additional_user_delay, connector, baseband_selector):
        """
        This function sets the feedback related settings.
        :param feedback_mode: Realtime feedback mode: Serial ('SER') or Serial 3x8 ('S3X8')
        :param serial_rate: Serial rate: 1.92 Mbps ('R1M9') 1.6 Mbps ('R1M6') or 115.2 Kbps ('R115')
        :param additional_user_delay: Additional user delay (in number of slots)
        :param connector: Feedback connector: Local ('LOC') or Global ('GLOB')
        :param baseband_selector: Baseband selector index
        :return: Instrument status
        """
        if feedback_mode is not None:
            self.io.write_str(f':BB:NR5G:TCW:RTF:MODE {feedback_mode}')
        if serial_rate is not None:
            self.io.write_str(f':BB:NR5G:TCW:RTF:SERR {serial_rate}')
        if additional_user_delay is not None:
            self.io.write_float(':BB:NR5G:TCW:RTF:AUSD ', additional_user_delay)
        if connector is not None:
            self.io.write_float(':BB:NR5G:TCW:RTF:CONN ', connector)
        if baseband_selector is not None:
            self.io.write_int(':BB:NR5G:TCW:RTF:BBS ', baseband_selector)
        return self.io.query_str(':syst:err?')

    def reset(self):
        """
        This function resets the connected instrument
        :return: Instrument status
        """
        self.io.write_str('*RST')
        return self.io.query_str(':syst:err?')

    def tcw_apply(self):
        """
        This function applies the signal configuration (TCW)
        :return: Instrument status
        """
        self.io.write_str_with_opc(':BB:NR5G:TCW:APPL')
        return self.io.query_str(':syst:err?')

    def output_on(self, output: int):
        """
        This function switches the output on
        :param output: Output number
        :return: Instrument status
        """
        self.io.write_str(f':OUTPut{output}:STAT 1')
        return self.io.query_str(':syst:err?')

    def output_off(self, output: int):
        """
        This function switches the output off
        :param output: Output number
        :return: Instrument status
        """
        self.io.write_str(f':OUTPut{output}:STAT 0')
        return self.io.query_str(':syst:err?')

    def close(self):
        """
        This function closes the instrument connection
        """
        self.io.close()

    def query_status(self):
        """
        This function queries the status of the connected instrument
        :return: Instrument status
        """
        return self.io.query_str(':syst:err?')

    def query_errors(self):
        """
        This function queries all errors of the connected instrument
        :return: List of errors
        """
        return self.io.query_all_errors()

    def query_awgn_power(self):
        """
        This function returns the AWGN power level in dBm
        :return: AWGN power level [dBm]
        """
        awgn_query = self.io.query_str(':BB:NR5G:TCW:AWGN:PLEV?').split(' ')
        awgn_power = float(awgn_query.pop(0)[1:])
        return awgn_power

    def query_awgn_bandwidth(self):
        """
        This function returns the AWGN bandwdith in MHz
        :return: AWGN bandwidht [MHz]
        """
        awgn_query = self.io.query_str(':BB:NR5G:TCW:AWGN:PLEV?').split(' ')
        awgn_bandwdith = float(awgn_query.pop(3))
        return awgn_bandwdith

    def query_fraction_max_throughput(self):
        """
        This function returns the  fraction of maximum throughput for the FRC
        :return: Fraction of maximum throughput
        """
        query_throughput = self.io.query_str(':BB:NR5G:TCW:WS:FMTH?')
        throughput = float(query_throughput.split('T').pop(1)) / 100
        return throughput

    def query_frc(self):
        """
        This function returns the used fixed reference channel
        :return: Fixed reference channel
        """
        return self.io.query_str(':BB:NR5G:TCW:WS:FRC:TYPE?')

    def query_power(self):
        """
        This function returns the power level in dBm
        :return: Power level [dBm]
        """
        return self.io.query_float(':BB:NR5G:TCW:WS:PLEV?')

    def query_timing_offset_base_value(self):
        """
        This function returns the timing offset base value in µs
        :return: Timing offset base value in µs
        """
        return self.io.query_float(':BB:NR5G:TCW:WS:TIOB?')

    def _check_device_suitability(self, needed: List[str]):
        """This function returns missing, required and already existing software options of the connected device."""
        missing = []
        for option in needed:
            if option not in self.io.instrument_options:
                missing.append(option)
        if len(missing) > 0:
            raise RsException(
                f'Some of the required options are missing.\nRequired: {", ".join(needed)}\n'
                f'Missing: {", ".join(missing)}\nAll options: {", ".join(self.io.instrument_options)}')
