from GFM315_Px_tests.General.GFM315_base import PxTestsBase
from GFM315_Px_tests.General.GFM315_utilities import RsException
from RsInstrument.Internal.InstrumentErrors import RsInstrException


class TC823(PxTestsBase):
    def __init__(self, resource_name: str):
        super().__init__(resource_name)
        super().reset()

        self.release = "REL16"
        """Select specification release ('REL15' or 'REL16')"""

        self.bs_class = None
        """Select base station class: Wide area ('WIDE'), medium area ('MED') or local area ('LOC')"""

        self.trigger_config = None
        """Select trigger configuration: Armed auto ('AAUT') or unchanged ('UNCH')"""
        self.marker_config = None
        """Select marker configuration: Radio frame start ('FRAM') or unchanged ('UNCH')"""

        self.tx_antennas = None
        """Set the number of transmit antennas"""
        self.rx_antennas = None
        """Set the number of receive antennas"""

        self.frequency = None
        """Set frequency in MHz"""
        self.duplex = None
        """Select duplex mode: Frequency division duplex ('FDD') or time division duplex ('TDD')"""
        self.channel_bw = None
        """Set channel bandwidth in MHz"""
        self.scs = None
        """Set sub carrier spacing in kHz"""
        self.cell_id = None
        """Set the cell id"""
        self.ue_id = None
        """Set the user equipment id"""
        self.prop_cond = None
        """Select the propagation conditions ('TDLB100D400' or 'TDLC300D100' or 'TDLA30D10')"""
        self.mapping_type = None
        """Select mapping type ('A' or 'B')"""
        self.dmrs_pos = None
        """Set the DMRS type a position"""
        self.rb_offset = None
        """Set the shift of the allocated wanted signal in number of resource blocks"""
        self.csi_part = None
        """Select the CSI part for the test case (1 or 2)"""
        self.uci_bits = None
        """Set the number of of UCI bits used (7 or 40)"""
        self.csi1_pattern = None
        """Set the CSI1 patter; Defines the frequency and time domain of the CSI part 1 subcarrier location;
        Please enter the pattern in the following format '10001....'"""
        self.csi2_pattern = None
        """Set the CSI2 patter; Defines the frequency and time domain of the CSI part 2 subcarrier location;
        Please enter the pattern in the following format '10001....'"""

        self._check_device_suitability(['K62', 'K144'])
        """Minimum software option requirement: Maybe for different setups (more antennas, etc.) additional options 
        are required """

    def apply_configuration(self):
        """
        This function applies the configuration and sends the remote control commands to the connected device.
        :return:
        """
        try:
            self.io.write_str(f':BB:NR5G:TCW:REL {self.release}')
            self.io.write_str(':BB:NR5G:TCW:TC TS381411_TC823')
            if self.bs_class is not None:
                self.io.write_str(':BB:NR5G:TCW:BSCL ' + self.bs_class)

            super().instrument_settings(self.trigger_config, self.marker_config)

            super().antenna_settings(self.tx_antennas, self.rx_antennas)

            if self.frequency is not None:
                self.io.write_float(':BB:NR5G:TCW:WS:RFFR ', self.frequency * 1000000)
            if self.duplex is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:DUPL {self.duplex}')
            if self.channel_bw is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:CBW BW{self.channel_bw}')
            if self.scs is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:SCSP N{self.scs}')
            if self.cell_id is not None:
                self.io.write_int(':BB:NR5G:TCW:WS:CELL', self.cell_id)
            if self.ue_id is not None:
                self.io.write_int(':BB:NR5G:TCW:WS:UEID', self.ue_id)
            if self.prop_cond is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:PROC {self.prop_cond}')
            if self.mapping_type is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:MAPT {self.mapping_type}')
            if self.dmrs_pos is not None:
                self.io.write_int(':BB:NR5G:TCW:WS:TAP', self.dmrs_pos)
            if self.rb_offset is not None:
                self.io.write_int(':BB:NR5G:TCW:WS:RBOF', self.rb_offset)

            # CSI part
            if self.csi_part is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:UCI:CSIPart CSIP_{self.csi_part}')
            if self.uci_bits is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:UCI:BITS B_{self.uci_bits}')
            if self.uci_bits == 7 or self.uci_bits is None:
                if self.csi1_pattern is not None:
                    if len(self.csi1_pattern) != 5 or list(set(self.csi1_pattern) - set('10')) != []:
                        return 'CSI1 pattern has not the right format'
                    else:
                        self.io.write_str(':BB:NR5G:TCW:WS:UCI:CSI1:PATT #H'
                                          + '{0:02X}'.format(int(self.csi1_pattern, 2)) + ",5")
                if self.csi2_pattern is not None:
                    if len(self.csi2_pattern) != 2 or list(set(self.csi2_pattern) - set('10')) != []:
                        return 'CSI2 pattern has not the right format'
                    else:
                        self.io.write_str(
                            ':BB:NR5G:TCW:WS:UCI:CSI2:PATT #H' + '{0:02X}'.format(int(self.csi2_pattern, 2)) + ",2")
            elif self.uci_bits == 40:
                if self.csi1_pattern is not None:
                    if len(self.csi1_pattern) != 20 or list(set(self.csi1_pattern) - set('10')) != []:
                        return 'CSI1 pattern has not the right format'
                    else:
                        self.io.write_str(':BB:NR5G:TCW:WS:UCI:CSI1:PATT #H' +
                                          '{0:05X}'.format(int(self.csi1_pattern, 2)) + ",20")
                if self.csi2_pattern is not None:
                    if len(self.csi2_pattern) != 20 or (list(set(self.csi2_pattern) - set('10')) != []):
                        return 'CSI2 pattern has not the right format'
                    else:
                        self.io.write_str(':BB:NR5G:TCW:WS:UCI:CSI2:PATT #H' +
                                          '{0:05X}'.format(int(self.csi2_pattern, 2)) + ",20")
            super().tcw_apply()
        except RsInstrException as e:
            raise RsException(e.args[0])
