from GFM315_Px_tests.General.GFM315_base import PxTestsBase
from GFM315_Px_tests.General.GFM315_utilities import RsException
from RsInstrument.Internal.InstrumentErrors import RsInstrException


class TC825(PxTestsBase):
    def __init__(self, resource_name: str):
        super().__init__(resource_name)
        super().reset()

        self.release = "REL16"
        """Select specification release ('REL15' or 'REL16')"""

        self.bs_class = None
        """Select base station class: Wide area ('WIDE'), medium area ('MED') or local area ('LOC')"""

        self.trigger_config = None
        """Select trigger configuration: Armed auto ('AAUT') or unchanged ('UNCH')"""
        self.marker_config = None
        """Select marker configuration: Radio frame start ('FRAM') or unchanged ('UNCH')"""

        self.tx_antennas = None
        """Set the number of transmit antennas"""
        self.rx_antennas = None
        """Set the number of receive antennas"""

        self.frequency = None
        """Set frequency in MHz"""
        self.duplex = None
        """Select duplex mode: Frequency division duplex ('FDD') or time division duplex ('TDD')"""
        self.channel_bw = None
        """Set channel bandwidth in MHz"""
        self.scs = None
        """Set sub carrier spacing in kHz"""
        self.cell_id = None
        """Set the cell id"""
        self.ue_id = None
        """Set the user equipment id"""
        self.testrequirement = None
        """Select the test requirement (normal mode 'NORM' or High Speed Train 'HST')"""
        self.prop_cond = None
        """Select the propagation conditions ('TDLC300D1200' or 'HST1NR350' or 'HST1NR500' or 'HST3NR350' or 
        'HST3NR500') """
        self.mappingtype = None
        """Select mapping type ('A' or 'B')"""
        self.transmitsrs = None
        """Transmit SRS (1 or 0)"""

        self.ue_id_stat = None
        """Set the user equipment id of stationary UE"""
        self.transmitsrs_stat = None
        """Transmit SRS (1 or 0) for stationary UE"""

        self.feedback_mode = None
        """Select the realtime feedback mode: Serial ('SER') or Serial 3x8 ('S3X8')"""
        self.serial_rate = None
        """Select serial rate: 1.92 Mbps ('R1M9') 1.6 Mbps ('R1M6') or 115.2 Kbps ('R115')"""
        self.additional_user_delay = None
        """Set additional user delay (in number of slots)"""
        self.connector = None
        """Select feedback connector: Local ('LOC') or Global ('GLOB') for moving UE"""
        self.baseband_selector = None
        """Set the baseband selector index of the moving UE"""
        self.baseband_selector_stat = None
        """Set the baseband selector index of the stationary UE"""
        self.connector_stat = None
        """Select feedback connector: Local ('LOC') or Global ('GLOB') for stationary UE"""

        self._check_device_suitability(['K62', 'K71', 'K144', 'K148', 'K145'])
        """Minimum software option requirement: Maybe for different setups (more antennas, etc.) additional options 
        are required """

    def apply_configuration(self):
        """
        This function applies the configuration and sends the remote control commands to the connected device.
        :return:
        """
        try:
            self.io.write_str(f':BB:NR5G:TCW:REL {self.release}')
            self.io.write_str(':BB:NR5G:TCW:TC TS381411_TC825')
            if self.bs_class is not None:
                self.io.write_str(f':BB:NR5G:TCW:BSCL {self.bs_class}')

            super().instrument_settings(self.trigger_config, self.marker_config)

            super().antenna_settings(self.tx_antennas, self.rx_antennas)

            if self.frequency is not None:
                self.io.write_float(':BB:NR5G:TCW:WS:RFFR ', self.frequency * 1000000)
            if self.duplex is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:DUPL {self.duplex}')
            if self.channel_bw is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:CBW BW{self.channel_bw}')
            if self.scs is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:SCSP N{self.scs}')
            if self.cell_id is not None:
                self.io.write_int(':BB:NR5G:TCW:WS:CELL', self.cell_id)
            if self.ue_id is not None:
                self.io.write_int(':BB:NR5G:TCW:WS:UEID', self.ue_id)
            if self.prop_cond is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:PROC {self.prop_cond}')
            if self.testrequirement is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:TREQuire  {self.testrequirement}')
            if self.mappingtype is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:MAPType {self.mappingtype}')
            if self.transmitsrs is not None:
                self.io.write_int(':BB:NR5G:TCW:WS:MUE:TSRS', self.transmitsrs)

            if self.ue_id_stat is not None:
                self.io.write_int(':BB:NR5G:TCW:IS:UEID', self.ue_id_stat)
            if self.transmitsrs_stat is not None:
                self.io.write_int(':BB:NR5G:TCW:SUE:TSRS', self.transmitsrs_stat)

            super().feedback_settings(self.feedback_mode, self.serial_rate, self.additional_user_delay, self.connector,
                                      self.baseband_selector)
            if self.connector_stat is not None:
                self.io.write_str(f':BB:NR5G:TCW:RTF:SUE:CONN {self.connector_stat}')
            if self.baseband_selector_stat is not None:
                self.io.write_str(f':BB:NR5G:TCW:RTF:SUE:BBSelector  {self.baseband_selector_stat}')

            super().tcw_apply()
        except RsInstrException as e:
            raise RsException(e.args[0])
